#include <utility.h>
#include <analysis.h>
#include <formatio.h>
#include <userint.h>
#include <stdio.h>
#include "hp34401.h"
#include "Proba.h" 

static int panelHandle;
ViSession  instrumentHandle;                           //Instrument handle
int        displayOn=1;   //inicijalno displej ukljucen na instrumentu
int        measureType=2;	   //inicijalno kao da je VDC merenje
int        range=1;	//range inicijalno automatski
ViReal64 measurementResults[100000];
ViReal64 timeDimension[100000];

/************************************************************************/
/* Poziv glavne funkcije (otvara pocetni prozor PANEL)                  */
/************************************************************************/
			  
int main(int argc, char *argv[])
   {
   if (InitCVIRTE (0, argv, 0) == 0)   
      return -1;    /* out of memory */
   if ((panelHandle = LoadPanel (0, "Proba.uir", PANEL)) < 0)
      return -1;
   DisplayPanel (panelHandle);
   //SetCtrlVal (panelHandle, PANEL_TEXTMSG, "");
   //SetCtrlVal (panelHandle, PANEL_TEXTMSG_2, "");
   RunUserInterface ();
   DiscardPanel (panelHandle);
   return 0;
   }

/************************************************************************/
/* Callback funkcija PowerOn (vrsi inicijalizaciju)                     */
/************************************************************************/

int CVICALLBACK PowerOn (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   int  state;
   ViReal64 manualRange;  
   double intTime; 
   switch (event)
      {
	  case EVENT_COMMIT:
	     {
	     GetCtrlVal (panelHandle, PANEL_KONTROLA, &state);
         switch (state) 
            {
            case 0 :
               {
			   hp34401_close (instrumentHandle); 
               SetCtrlAttribute (panelHandle, PANEL_VISEMERENJA, ATTR_DIMMED, 1);
			   SetCtrlAttribute (panelHandle, PANEL_JEDNOMERENJE, ATTR_DIMMED, 1);
			   SetCtrlAttribute (panelHandle, PANEL_RESET, ATTR_DIMMED, 1);
			   SetCtrlAttribute (panelHandle, PANEL_CITANJEGRESKE, ATTR_DIMMED, 1);
			   SetCtrlAttribute (panelHandle, PANEL_SELFTEST, ATTR_DIMMED, 1);
			   SetCtrlAttribute (panelHandle, PANEL_ABOUT, ATTR_DIMMED, 1);
			   SetCtrlAttribute (panelHandle, PANEL_ACV, ATTR_DIMMED, 1);
			   SetCtrlAttribute (panelHandle, PANEL_DCV, ATTR_DIMMED, 1);
			   SetCtrlAttribute (panelHandle, PANEL_R2W, ATTR_DIMMED, 1);
			   SetCtrlAttribute (panelHandle, PANEL_BUSTRIGGER, ATTR_DIMMED, 1);
			   SetCtrlAttribute (panelHandle, PANEL_EKRAN, ATTR_DIMMED, 1);
			   SetCtrlAttribute (panelHandle, PANEL_BROJSEMPLOVA, ATTR_DIMMED, 1);
			   SetCtrlAttribute (panelHandle, PANEL_BROJTRIGGERA, ATTR_DIMMED, 1);
			   break;
               }
            case 1 :
               {
			   hp34401_init ("GPIB::15", VI_TRUE, VI_TRUE, &instrumentHandle);
               SetCtrlAttribute (panelHandle, PANEL_VISEMERENJA, ATTR_DIMMED, 0);
			   SetCtrlAttribute (panelHandle, PANEL_JEDNOMERENJE, ATTR_DIMMED, 0);
			   SetCtrlAttribute (panelHandle, PANEL_RESET, ATTR_DIMMED, 0);
			   SetCtrlAttribute (panelHandle, PANEL_CITANJEGRESKE, ATTR_DIMMED, 0);
			   SetCtrlAttribute (panelHandle, PANEL_SELFTEST, ATTR_DIMMED, 0);
			   SetCtrlAttribute (panelHandle, PANEL_ABOUT, ATTR_DIMMED, 0);
			   SetCtrlAttribute (panelHandle, PANEL_ACV, ATTR_DIMMED, 0);
			   SetCtrlAttribute (panelHandle, PANEL_DCV, ATTR_DIMMED, 0);
			   SetCtrlAttribute (panelHandle, PANEL_R2W, ATTR_DIMMED, 0);
			   SetCtrlAttribute (panelHandle, PANEL_BUSTRIGGER, ATTR_DIMMED, 0);
			   SetCtrlAttribute (panelHandle, PANEL_EKRAN, ATTR_DIMMED, 0);
			   SetCtrlAttribute (panelHandle, PANEL_BROJSEMPLOVA, ATTR_DIMMED, 0);
			   SetCtrlAttribute (panelHandle, PANEL_BROJTRIGGERA, ATTR_DIMMED, 0);
			   
			   
			   
			   hp34401_confVoltDc(instrumentHandle); //konfigurisanje da se meri DC 
		       if(range)
                  {
                  hp34401_voltDcRangAuto(instrumentHandle,VI_TRUE);
                  }
               else
                  {
                  GetCtrlVal (panelHandle, PANEL_RUCNIOPSEG1, &manualRange);//Uzima manuelni opseg
                  hp34401_voltDcRang(instrumentHandle,manualRange); 
                  }
               GetCtrlVal (panelHandle, PANEL_INTTIME, &intTime);//Uzima vreme semplovanja
               hp34401_voltDcNplc(instrumentHandle, intTime);
			   break;
            }  }
		 break;
	  }  }
   return 0;
   }

/************************************************************************/
/* Callback funkcija Shutdown (gasi program)                            */
/************************************************************************/

int CVICALLBACK Shutdown (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   if (event == EVENT_COMMIT)
      QuitUserInterface(0);
   return(0);
   }

/************************************************************************/
/* Callback funkcija ACMeasure (merenje AC napona)                      */
/************************************************************************/

int CVICALLBACK ACMeasure (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   ViReal64 manualRange;
   switch (event)
      {
	  case EVENT_COMMIT:
	     {
         hp34401_confVoltAc(instrumentHandle); //konfigurisanje da se meri AC
		 if(range)
            {
            hp34401_voltAcRangAuto(instrumentHandle,VI_TRUE); //ukljucenje auto opsega
            }
         else
		    { 
            GetCtrlVal (panelHandle, PANEL_RUCNIOPSEG1, &manualRange);//Uzima manuelni opseg
            hp34401_voltAcRang(instrumentHandle,manualRange); //slanje  manuelnog opsega
            }
         SetCtrlAttribute (panelHandle, PANEL_VISEMERENJA, ATTR_DIMMED, 1);
		 measureType=1;
		 break;
	  }  }
	return 0;
   }

/************************************************************************/
/* Callback funkcija DCMeasure (merenje DC napona)                      */
/************************************************************************/

int CVICALLBACK DCMeasure (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   double intTime;
   ViReal64 manualRange;
   switch (event)
      {
      case EVENT_COMMIT:
         {
         hp34401_confVoltDc(instrumentHandle); //konfigurisanje da se meri AC 
		 if(range)
            {
            hp34401_voltDcRangAuto(instrumentHandle,VI_TRUE);
            }
         else
            {
            GetCtrlVal (panelHandle, PANEL_RUCNIOPSEG1, &manualRange);//Uzima manuelni opseg
            hp34401_voltDcRang(instrumentHandle,manualRange); 
            }
         GetCtrlVal (panelHandle, PANEL_INTTIME, &intTime);//Uzima vreme semplovanja
         hp34401_voltDcNplc(instrumentHandle, intTime);
         SetCtrlAttribute (panelHandle, PANEL_VISEMERENJA, ATTR_DIMMED, 0);
		 measureType=2;  
		 break;
	  }  }
   return 0;
   }

/************************************************************************/
/* Callback funkcija DisplayOnOff (prikaz na ekranu)                    */
/************************************************************************/

int CVICALLBACK DisplayOnOff (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   switch (event)
      {
	  case EVENT_COMMIT:
	     {
		 if (displayOn)
		    {
		    displayOn=0;
		    hp34401_dispStat(instrumentHandle, hp34401_Disp_Stat_off);
		    }
		 else 
		    {
		    displayOn=1;
		    hp34401_dispStat(instrumentHandle, hp34401_Disp_Stat_on);
		    }
		 break;
	  }  }
   return 0;
   }

/************************************************************************/
/* Callback funkcija Res2WMeasure (merenje otpornosti na 2 zice)        */
/************************************************************************/

int CVICALLBACK Res2WMeasure (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   double intTime;
   ViReal64 manualRange;
   switch (event)
      {
	  case EVENT_COMMIT:
	     {
         hp34401_confRes(instrumentHandle);
		 if(range)
            {
            hp34401_resRangAuto(instrumentHandle,VI_TRUE);
            }
         else
            {
            GetCtrlVal (panelHandle, PANEL_RUCNIOPSEG2, &manualRange);//Uzima manuelni opseg
            hp34401_resRang(instrumentHandle,manualRange);
            }
         GetCtrlVal (panelHandle, PANEL_INTTIME, &intTime);//Uzima vreme integracije
         hp34401_resNplc(instrumentHandle, intTime);
         SetCtrlAttribute (panelHandle, PANEL_VISEMERENJA, ATTR_DIMMED, 1);
		 measureType=3; 
		 break;
	  }  }
   return 0;
   }

/************************************************************************/
/* Callback funkcija ClearDisplay ( brise display)                         */
/************************************************************************/

int CVICALLBACK ClearDisplay (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   switch (event)
      {
	  case EVENT_COMMIT:
	     {
		 SetCtrlVal (panelHandle, PANEL_PORUKA2,""); 
		 SetCtrlVal (panelHandle, PANEL_TEXTMSG,"");
	     break;
	  }  }
   return 0;
   }

/************************************************************************/
/* Callback funkcija SingleMeas (jedno merenje)                          */
/************************************************************************/

int CVICALLBACK SingleMeas (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   ViReal64 value;
   char str[100];
   ViInt32  measurementNumber;
   int  triggerType;
   switch (event)
      {
	  case EVENT_COMMIT:
	     {
		 
		 DeleteGraphPlot (panelHandle, PANEL_GRAPH, -1, VAL_IMMEDIATE_DRAW);//brise grafik
		 
		 
		 hp34401_timeOut (instrumentHandle, 100000);//IO timeout u ms
		 hp34401_autoZero(instrumentHandle, VI_TRUE);//podesavanje nule automatsko
		 
		 GetCtrlVal (panelHandle, PANEL_TRIGGER, &triggerType);//preuzimanje vrste trigera
		 hp34401_trigSour(instrumentHandle,triggerType); //Izbor trigera
		 
		 
		 hp34401_sampCoun(instrumentHandle,1); //Broj semplova=1
		 hp34401_trigCoun(instrumentHandle,1); //Broj trigera =1
		 hp34401_trigDelAuto(instrumentHandle, VI_FALSE); //Auto Delay iskljucen
		 
		 
		 
		 
		 
		 
		 hp34401_initImm(instrumentHandle);//Pocetak merenja, to jest cekanja na triger
		 
		 
		 if (triggerType==2) //triger je immediate, merenje odmah
		    {
		    hp34401_fetc_Q(instrumentHandle, &value, &measurementNumber); 
		    Fmt (str, "%f", value);
		    SetCtrlVal (panelHandle, PANEL_TEXTMSG, str);
		    } 
		 else  //trigger je spoljasnji ili preko busa
		    {
		    SetCtrlVal (panelHandle, PANEL_TEXTMSG, "Watch Instr");
		    }
		 switch (measureType) //ispis jedinice
		    {
		    case 1:
		       {
		       SetCtrlVal (panelHandle, PANEL_PORUKA2, "VAC");
		       break;
		       }
		    case 2:
		       {
		       SetCtrlVal (panelHandle, PANEL_PORUKA2, "VDC");
		       break; 
		       }
		    case 3:
		       {
		       SetCtrlVal (panelHandle, PANEL_PORUKA2, "OMA");
		       break; 
		    }  }
		 break;
	  }  }
   return 0;
   }

/************************************************************************/
/*       Callback funkcija MultiMeas (vise merenja)                     */
/************************************************************************/

int CVICALLBACK MultiMeas (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   ViInt32  triggerCount;
   ViInt32  sampleCount;
   ViInt32  measurementNumber;
   double intTime,measurementInterval, rmsValue, t1, t2;
   int i;
   char str[100]="Only DCV";
   switch (event)
      {
	  case EVENT_COMMIT:
	     {
		 if(measureType==2)//ako je merenje postavljeno na DC
		    {
		    SetCtrlVal (panelHandle, PANEL_PORUKA2, " "); 
		    SetCtrlVal (panelHandle, PANEL_TEXTMSG, " ");
		    
			GetCtrlVal (panelHandle, PANEL_BROJSEMPLOVA, &sampleCount); 
		    GetCtrlVal (panelHandle, PANEL_BROJTRIGGERA, &triggerCount);
		    
			hp34401_sampCoun(instrumentHandle, sampleCount);
		    
			hp34401_trigSour(instrumentHandle, hp34401_TRIG_SOUR_IMM);
			
			hp34401_trigCoun(instrumentHandle, triggerCount);
		    
			
			hp34401_trigDelAuto(instrumentHandle, VI_TRUE);//ukljucuje se auto trigger delay
		    hp34401_timeOut (instrumentHandle, 100000);//timeout
			
			
			
			t1=Timer();//preuzimam vreme pocetka
		    hp34401_initImm(instrumentHandle);//Pocetak merenja
		    //t1=Timer();
		    hp34401_fetc_Q(instrumentHandle, measurementResults, &measurementNumber);//prebacivanje podataka
		    t2=Timer();//preuzimam vreme kraja merenja
            
			
			
			measurementInterval=((t2-t1)*1000);//vreme izvrsavanja funkcije hp34401_fetc_Q (vreme merenja) realno vreme merenja je manje za vreme slanja podataka itd..
			
			
			SetCtrlVal (panelHandle, PANEL_VREMEMERENJA,measurementInterval);
			
			//trajanje jednog merenja:
			intTime=measurementInterval/(measurementNumber-1);
		    //priprema x ose
			for (i=0;i<= measurementNumber;i++)
		       {
		       timeDimension[i]=i*intTime;
		       }
		    
			
			
			DeleteGraphPlot (panelHandle, PANEL_GRAPH, -1, VAL_IMMEDIATE_DRAW);
			
			
			PlotXY (panelHandle, PANEL_GRAPH, timeDimension, measurementResults, measurementNumber, VAL_DOUBLE,VAL_DOUBLE, VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1,VAL_YELLOW);
			
			
			
			
			RMS (measurementResults,measurementNumber,&rmsValue);
		    
			Fmt (str, "%f", rmsValue);
		    SetCtrlVal (panelHandle, PANEL_TEXTMSG, str); 
		    SetCtrlVal (panelHandle, PANEL_PORUKA2, "RMS");
			}
		 else 
		    {
			DeleteGraphPlot (panelHandle, PANEL_GRAPH, -1, VAL_IMMEDIATE_DRAW);
		    SetCtrlVal (panelHandle, PANEL_PORUKA2, " "); 
		    SetCtrlVal (panelHandle, PANEL_TEXTMSG, " ");
			SetCtrlVal (panelHandle, PANEL_TEXTMSG, str);//ispis da mora da bude VDC da bi radio multMeas
			}
		 break;
	  }  }
   return 0;
   }

/************************************************************************/
/*       Callback funkcija Reset                                        */
/************************************************************************/

int CVICALLBACK Reset (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   switch (event)
      {
	  case EVENT_COMMIT:
	     {
		 hp34401_reset (instrumentHandle); //resetovanje
		 break;
	  }  }
   return 0;
   }

/************************************************************************/
/*       Callback funkcija ReadError                                    */
/************************************************************************/

int CVICALLBACK ReadError (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   ViInt32 errorCode;
   ViChar errorMessage[100];
   switch (event)
      {
	  case EVENT_COMMIT:
	     {
		 
			 
		 hp34401_error_query(instrumentHandle,&errorCode,errorMessage);//trazi se informacija o gresci
		 
		 
		 SetCtrlVal (panelHandle, PANEL_TEXTMSG, errorMessage); 
		 SetCtrlVal (panelHandle, PANEL_PORUKA2, ""); 
		 break;
	  }  }
   return 0;
   }

/************************************************************************/
/*       Callback funkcija Range                                    */
/************************************************************************/

int CVICALLBACK Range (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   switch (event)
      {
	  case EVENT_COMMIT:
	     {
         if(range)
            {
            range=0;
            }
         else range=1;  
		 break;
	  }  }
   return 0;
   }

/************************************************************************/
/*       Callback funkcija SelfTest                                    */
/************************************************************************/

int CVICALLBACK SelfTest (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   ViChar  testMessage[256];
   ViInt16 testResult;
   switch (event)
      {
	  case EVENT_COMMIT:
	     {
		 
			 
		 hp34401_timeOut(instrumentHandle,100000);//timeout postavljen na 100 sec
		 
		 
		 hp34401_self_test (instrumentHandle,&testResult,testMessage);
		 
		 
 		 SetCtrlVal (panelHandle, PANEL_TEXTMSG, testMessage);
		 SetCtrlVal (panelHandle, PANEL_PORUKA2, "");      
		 break;
	  }  }
   return 0;
   }

/************************************************************************/
/*       Callback funkcija InstrumentDriver                             */
/************************************************************************/

int CVICALLBACK InstrumentDriver (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   ViChar driver_rev[256];
   ViChar instr_rev[256];
   switch (event)
      {
	  case EVENT_COMMIT:
	     {
		 hp34401_revision_query(instrumentHandle, driver_rev, instr_rev);
		 SetCtrlVal (panelHandle, PANEL_TEXTMSG, driver_rev);
		 SetCtrlVal (panelHandle, PANEL_PORUKA2, instr_rev);
	     break;
	  }  }
   return 0;
   }

/************************************************************************/
/*       Callback funkcija BusTrigg                                     */
/************************************************************************/

int CVICALLBACK BusTrigger (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   switch (event)
      {
   	  case EVENT_COMMIT:
   	     {
         hp34401_trg(instrumentHandle);
         break;
      }  }
   return 0;
   }
