#include <utility.h>
#include "hp34401.h"
#include <cvirte.h>		
#include <userint.h>
#include "gpib.h"

static int panelHandle;
int state;
ViSession  instrumentHandle;                           //Instrument handle
ViStatus errStatus; 


int main (int argc, char *argv[])
   {
   if (InitCVIRTE (0, argv, 0) == 0)
		return -1;	/* out of memory */
   if ((panelHandle = LoadPanel (0, "gpib.uir", PANEL)) < 0)
		return -1;
   DisplayPanel (panelHandle);
	
   RunUserInterface ();
   DiscardPanel (panelHandle);
   return 0;
   }

int CVICALLBACK close (int panel, int event, void *callbackData,int eventData1, int eventData2)
   {
   switch (event)
      {
	  case EVENT_CLOSE:
	     QuitUserInterface (0);
		 break;
	  }
   return 0;
   }

int CVICALLBACK prekidac (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   char disp[200];
   switch (event)
      {
	  case EVENT_COMMIT:
	  	 {
		 
			 
	     GetCtrlVal (panelHandle, PANEL_BINARYSWITCH, &state);//preuzima se nova vrednost prekidaca
         
		 
		 
		 switch (state) 
            {
            case 0: //prekidac na OFF
	           {   
			   errStatus = hp34401_close (instrumentHandle); //pokusaj zatvaranja veze sa instrumentom
		       SetCtrlVal (panelHandle, PANEL_VREDNOST, 0.00);//prebacivanje prikaza na voltmetru na 0.00 (ugasen)
			   hp34401_error_message(instrumentHandle,errStatus,disp); //konverzija koda statusa operacije errStatus
               
			   //u string disp 		       
		       if (VI_SUCCESS > errStatus)//ako je kod statusa negativan, javila se greska
			      {
				  MessagePopup("Error",disp);//ispisuje se poruka o gresci
				  }
			   break; 
			   }
			
			   
		    case 1: //prekidac na VDC
		       {
		       errStatus = hp34401_init ("GPIB0::15::INSTR",1,1, /*VI_TRUE, VI_TRUE,*/ &instrumentHandle);
			   //pokusaj otvaranja veze sa instrumentom
		       hp34401_error_message(instrumentHandle,errStatus,disp);//konverzija koda statusa operacije errStatus
			   														  //u string disp
			   if (VI_SUCCESS > errStatus) //ako je kod statusa negativan, javila se greska
			      {
				  SetCtrlVal (panelHandle, PANEL_VREDNOST, 0.00);//prebacivanje prikaza na voltmetru na 0.00 (ugasen)		  
				  state=0;
				  SetCtrlVal (panelHandle, PANEL_BINARYSWITCH, state);//prebacivanje preklopnika na OFF
				  MessagePopup("",disp);
				  }
			   else	   //veza otvorena
			      {
				  
				  //podesavanje instrumenta da meri DC Voltage
				  hp34401_confVoltDc(instrumentHandle); 
				  
				  //ukljucenje auto opsega, nakon ukljucenja je svakako u ovom stanju
                  hp34401_voltDcRangAuto(instrumentHandle,VI_TRUE); 
				  
				  //vreme integracije u periodima mreznog napona, nakon ukljucenja 10.00
                  hp34401_voltDcNplc(instrumentHandle, 0.02);	
				  
				  
				  
				  
				  //time out za IO na magistrali u ms, po defaultu je 2s
			      hp34401_timeOut (instrumentHandle, 100000);
				  
				  //automatsko podesavanje nule, nakon reseta je svakako isto ovako
				  hp34401_autoZero(instrumentHandle, VI_TRUE);
				  
		    	  //izvor trigera, nakon reseta je isto ovako, trenutno
				  hp34401_trigSour(instrumentHandle,2);//immediate trigger
				  
				  //broj merenja po trigeru, nakon reseta je isto ovako
		          hp34401_sampCoun(instrumentHandle,1); //Broj semplova=1
				  
				  //broj trigera koji se salje, nakon reseta je isto ovako
		          hp34401_trigCoun(instrumentHandle,1); //Broj trigera =1
				  
				  //koriscenje auto delaya, po resetu je ukljucen
		          hp34401_trigDelAuto(instrumentHandle, VI_FALSE); //Auto Delay nema
				  
				  
				  
				  
				  //perioda timera 0.1s
				  SetCtrlAttribute (panelHandle, PANEL_TIMER, ATTR_INTERVAL, 0.1);
				  
				  
				  }
			   break;
			   }
			case 2: //prekidac na VAC, slicno kao VDC ali nema vreme integracije
		       {
		       errStatus =hp34401_init ("GPIB0::15::INSTR",1,1, /*VI_TRUE, VI_TRUE,*/ &instrumentHandle);
		       hp34401_error_message(instrumentHandle,errStatus,disp);
			   if (VI_SUCCESS > errStatus)
			      {
				  SetCtrlVal (panelHandle, PANEL_VREDNOST, 0.00);		  
				  state=0;
				  SetCtrlVal (panelHandle, PANEL_BINARYSWITCH, state);
				  MessagePopup("",disp);
				  }  
			   else
			      {
				  
					  
					  
				  hp34401_confVoltAc(instrumentHandle);
                  hp34401_voltAcRangAuto(instrumentHandle,VI_TRUE);
			      
				  
				  
				  
				  hp34401_timeOut (instrumentHandle, 100000);
				  hp34401_autoZero(instrumentHandle, VI_TRUE);
		    	  hp34401_trigSour(instrumentHandle,2);//immediate trigger
		          hp34401_sampCoun(instrumentHandle,1); //Broj semplova=1
		          hp34401_trigCoun(instrumentHandle,1); //Broj trigera =1
		          hp34401_trigDelAuto(instrumentHandle, VI_FALSE); //Auto Delay nema
				  
				  
				  //perioda timera 3s, vreme merenja VAC je reda sekunde, bez ovoga program "stuca"
				  
				  SetCtrlAttribute (panelHandle, PANEL_TIMER, ATTR_INTERVAL, 3.00);
				  
				  }
			   break;
			   }
			case 3: //otpornost, skoro isto kao VDC
		       {
		       errStatus =hp34401_init ("GPIB0::15::INSTR",1,1, /*VI_TRUE, VI_TRUE,*/ &instrumentHandle);
		       hp34401_error_message(instrumentHandle,errStatus,disp);
			   if (VI_SUCCESS > errStatus)
			      {
				  SetCtrlVal (panelHandle, PANEL_VREDNOST, 0.00);		  
				  state=0;
				  SetCtrlVal (panelHandle, PANEL_BINARYSWITCH, state);
				  MessagePopup("",disp);
				  }
			   else
			      {
				  hp34401_confRes(instrumentHandle); 
                  hp34401_resRangAuto(instrumentHandle,VI_TRUE); 
                  hp34401_resNplc(instrumentHandle, 0.02); 
				  
				  
				  
				  
			      hp34401_timeOut (instrumentHandle, 100000);
				  hp34401_autoZero(instrumentHandle, VI_TRUE);
		    	  hp34401_trigSour(instrumentHandle,2);//immediate trigger
		          hp34401_sampCoun(instrumentHandle,1); //Broj semplova=1
		          hp34401_trigCoun(instrumentHandle,1); //Broj trigera =1
		          hp34401_trigDelAuto(instrumentHandle, VI_FALSE); //Auto Delay nema		          
				  
				  
				  SetCtrlAttribute (panelHandle, PANEL_TIMER, ATTR_INTERVAL, 0.1);
				  
				  }
			   break;
			}  }
		 break;
	     }
      }	
   return 0;
   }


int CVICALLBACK otkucaj (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
   {
   ViReal64 value;
   ViInt32 measurementNumber;
   switch (event)
      {
  	  case EVENT_TIMER_TICK:
	     {
		 if (state!=0)//sta god da se meri prikazuje se na indikatoru i grafiku
		    {
			
				
				hp34401_initImm(instrumentHandle);//Pocetak merenja
				
		        
				
				hp34401_fetc_Q(instrumentHandle, &value, &measurementNumber);//preuzimanje podataka
				
			    
				SetCtrlVal (panelHandle, PANEL_VREDNOST, value);
			   //tacka na stripchart
			   PlotStripChartPoint (panelHandle, PANEL_STRIPCHART, value);
			}
		 break;
	     }
      }
   return 0;
   }

